#! /usr/bin/env python
# -*- coding: utf-8 -*-
################################################################################
# Copyright (c) 2013, Mission Machines, Inc. All rights reserved.
# http://www.missionmachines.com
#
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions are met: 
#
# - Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the following disclaimer. 
# - Redistributions in binary form must reproduce the above copyright notice, 
#   this list of conditions and the following disclaimer in the documentation 
#   and/or other materials provided with the distribution. 
# - Neither name of Mission Machines, Inc. or the names of its contributors 
#   may be used to endorse or promote products derived from this software 
#   without specific prior written permission.
# 
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
#   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL MISSION MACHINES, Inc,
#   OR * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
#   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
#   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
#   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
#   OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#
#	Questions: andrew@missionmachines.com or support@missionmachines.com
#
__VERSION__ = "1.0.0"

import indigo
import socket
import re
import os
import sys
import time
import midoor
import time
import indigoPluginUpdateChecker


# Note the "indigo" module is automatically imported and made available inside
# our global name space by the host process.

################################################################################
class Plugin(indigo.PluginBase):
	DoorStatusString = { midoor.DoorStatus.Unknown: u"Unknown",
						 midoor.DoorStatus.Opened: u"Opened",
						 midoor.DoorStatus.Closed: u"Closed" }
						 
	########################################
	def __init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs):
		indigo.PluginBase.__init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs)
		self.debug = pluginPrefs.get("showDebugInfo", False)
		#default poll time 
		self.pollTimeSeconds = 60
		self.updater = indigoPluginUpdateChecker.updateChecker(self, 'http://www.missionmachines.net/modules/indigo/miDoorVersion.htm', 7)
		self.shutdown = False
		__VERSION__ = str(pluginVersion)
		self.debugLog(u"miDoor Indigo Plugin Version: "+__VERSION__)
		self.debugLog(u"Copyright (C) 2014 by Mission Machines, Inc.")

	def __del__(self):
		indigo.PluginBase.__del__(self)

	########################################
	def startup(self):
		pass

	def shutdown(self):
		self.shutdown = True
		pass

	########################################
	# If runConcurrentThread() is defined, then a new thread is automatically created
	# and runConcurrentThread() is called in that thread after startup() has been called.
	#
	# runConcurrentThread() should loop forever and only return after self.stopThread
	# becomes True. If this function returns prematurely then the plugin host process
	# will log an error and attempt to call runConcurrentThread() again after several seconds.
	def runConcurrentThread(self):
		try:
			while self.shutdown is False:
				self.updater.checkVersionPoll()
				wait = 2
				devs = self.getAllmiDoors()
				for x in devs:
					dev = x['device']
					self.debugLog(dev.name)
					# if if it is time
					poll = x['poll']
					pollVal = int(poll)
					if pollVal < 1 or pollVal> 4*60*60:
						pollVal = self.pollTimeSeconds
					try:
						lastUpdate = dev.pluginProps["lastUpdate"]
					except:
						self.debugLog("  No last")
						lastUpdate = time.time() - pollVal-1 
					now = time.time()
					#self.debugLog("  Last: " + str(lastUpdate) + " now: "+str(now) + " = " + str((now-lastUpdate)) + " vs " + str(pollVal))
					if (now-lastUpdate) < pollVal:
						self.debugLog("  Skipping: " + dev.name)
						continue	
					self.debugLog("  Go")
					addr = x['address']
					user = x['user']		
					passwd = x['password']		
					port = x['port']		
					toVal = x['to']
					door = midoor.miDoor(addr, user, passwd, int(port))
					door.SetTimeout(toVal)
					rc = door.StartUpdateClose()
					self.debugLog("  " + str(rc))
					dev.updateStateOnServer('doorState', self.DoorStatusString[door.GetDoorStatus()] )
					dev.updateStateOnServer('celsius', door.GetTemperature(), 1)
					if door.GetTemperature() < -400.0:
						# uninitialized (i.e. unknown temp)
						dev.updateStateOnServer('fahrenheit', door.GetTemperature(), 1)
					else:
						dev.updateStateOnServer('fahrenheit', (door.GetTemperature()*9.0)/5.0 + 32.0, 1)
					dev.updateStateOnServer('isAlarmed', door.GetAlarmed() > midoor.AlarmTrigger.Reserved)
					msg = door.GetAlarmMessage()
					if msg == None:
						msg = "No Alarm"
					dev.updateStateOnServer('alarm', msg)
					try:
						now = time.time()
						self.debugLog("  Setting time: " + str(now))
						copy = dev.pluginProps
						copy.update({"lastUpdate":now})
						dev.replacePluginPropsOnServer(copy)
					except:
						self.debugLog("  Update time missed")
						pass
				self.sleep(wait)
		except self.StopThread:
			pass	# Optionally catch the StopThread exception and do any needed cleanup.

	# Get an array of all devices and load prefs
	def getAllmiDoors(self):
		devs = []
		for x in indigo.devices.iter("self"):
			try:
				dev = {}
				dev['address'] = x.pluginProps['address']
				dev['user'] = x.pluginProps['user']
				dev['password'] = x.pluginProps['password']
				dev['port'] = x.pluginProps['port']
				dev['poll'] = x.pluginProps['poll']
				to = x.pluginProps['timeout']
				toVal = int(to)
				if toVal < 1 or toVal > 30:
					toVal = 4
				dev['to'] = toVal * 1000
				dev['id'] = x.id
				dev['device'] = x
				devs.append(dev)
			except:
				pass
		return devs
	
	# Get the device prefs for the device with the given id			
	def findDevice(self, id):
		devs = self.getAllmiDoors()
		for x in devs:
			if x['id'] == id:
				return x
		return None
		
	########################################
	# Actions defined in MenuItems.xml:
	########################################
	def clearAlarms(self):
		devs = self.getAllmiDoors()
		for x in devs:
			self.debugLog(u"Clear Alarm: " + x['address'])
			addr = x['address']
			user = x['user']		
			passwd = x['password']	
			port = x['port']		
			door = midoor.miDoor(addr, user, passwd, int(port))
			door.SetTimeout(x['to'])
			rc = door.StartClearAlarmClose()
			
	def checkUpdateNow(self):
		if self.updater != None:
			self.updater.checkVersionNow()
		
	########################################
	def validateDeviceConfigUi(self, valuesDict, typeId, devId):
		errorDict = indigo.Dict()
		rc = True
		if typeId == "miDoor":
			address = valuesDict['address']
			port = valuesDict['port']
			passwd = valuesDict['password']
			user = valuesDict['user']
			poll = valuesDict['poll']
			to = valuesDict['timeout']
			#validate user
			if len(user) <= 0 or len(user) > 64:
				rc = False
				errorDict['user'] = u"Username must be specified and proper length"
			#validate password
			if len(passwd) <= 0 or len(passwd) > 64:
				rc = False
				errorDict['password'] = u"Password must be specified and proper length"
			#Validate Port
			portVal = int(port)
			if portVal < 1 or portVal > 65535:
				rc = False
				errorDict['port'] = u"App Server Port must be in the range of 1 to 65535.  Default is 2553"
			#validate poll time
			pollVal = int(poll)
			if pollVal < 10 or pollVal > 4*60*60:
				rc = False
				errorDict['poll'] = u"Poll Time must be from 10 seconds to 14400 seconds (4 Hours).  Default is 60 seconds"
			#validate connection timeout
			toVal = int(to)
			if toVal < 2 or toVal > 30:
				rc = False
				errorDict['timeout'] = u"Connection Timeout must be from 2 seconds to 30 seconds.  Default is 4 seconds"
			#Validate Address
			#Look for blatant dotted notation errors
			pat = re.compile("^\d{1,8}\.\d{1,8}\.\d{1,8}\.\d{1,8}$")
			if pat.match(address):
				#Check dotted IP Address
				try:
					socket.inet_aton(address)
				except socket.error:
					rc = False
					errorDict["address"] = u"Invalid IP Address"
			else:
				if address == None or len(address) < 1 or len(address) > 255:
					rc = False
					errorDict["address"] = u"Invalid Hostname"
				pass
		return (rc, valuesDict, errorDict)

	########################################
	# Plugin Actions object callbacks (pluginAction is an Indigo plugin action instance)
	########################################
	
	def pushCallback(self, action):
		x = self.findDevice(action.deviceId)
		if x != None:
			self.debugLog(u"Push Button")
			addr = x['address']
			user = x['user']		
			passwd = x['password']	
			port = x['port']		
			door = midoor.miDoor(addr, user, passwd, int(port))
			door.SetTimeout(x['to'])
			door.StartPushClose()
		
	def clearAlarm(self, action):
		x = self.findDevice(action.deviceId)
		if x != None:
			self.debugLog(u"Clear Alarm")
			addr = x['address']
			user = x['user']		
			passwd = x['password']	
			port = x['port']		
			door = midoor.miDoor(addr, user, passwd, int(port), self)
			door.SetTimeout(x['to'])
			door.StartClearAlarmClose()
